import { EventEmitter, Directive, ElementRef, Renderer2, Input, Output, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const Mark = require('mark.js');
/** @type {?} */
let cancelAnimationId;
/**
 * @param {?} __0
 * @return {?}
 */
function animate({ timing, draw, duration }) {
    /** @type {?} */
    const start = performance.now();
    cancelAnimationId = requestAnimationFrame((/**
     * @param {?} time
     * @return {?}
     */
    function animate2(time) {
        // timeFraction goes from 0 to 1
        /** @type {?} */
        let timeFraction = (time - start) / duration;
        if (timeFraction > 1) {
            timeFraction = 1;
        }
        // calculate the current animation state
        /** @type {?} */
        const progress = timing(timeFraction);
        draw(progress); // draw it
        if (timeFraction < 1) {
            cancelAnimationId = requestAnimationFrame(animate2);
        }
    }));
}
class MarkjsHighlightDirective {
    /**
     * @param {?} contentElementRef
     * @param {?} renderer
     */
    constructor(contentElementRef, renderer) {
        this.contentElementRef = contentElementRef;
        this.renderer = renderer;
        this.markjsHighlight = '';
        this.markjsConfig = {};
        this.minimumMarkLength = 3;
        this.scrollToFirstMarked = false;
        this.getInstance = new EventEmitter();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (!this.markInstance) {
            this.markInstance = new Mark(this.contentElementRef.nativeElement);
            this.getInstance.emit(this.markInstance);
        }
        this.hightlightText();
        if (this.scrollToFirstMarked) {
            this.scrollToFirstMarkedText();
        }
    }
    /**
     * @return {?}
     */
    hightlightText() {
        this.markjsHighlight = this.markjsHighlight || '';
        if (this.markjsHighlight && this.markjsHighlight.length <= (this.minimumMarkLength - 1)) {
            this.markInstance.unmark();
            return;
        }
        else {
            this.markInstance.unmark({
                done: (/**
                 * @return {?}
                 */
                () => {
                    this.markInstance.mark((this.markjsHighlight || ''), this.markjsConfig);
                })
            });
        }
    }
    /**
     * @return {?}
     */
    scrollToFirstMarkedText() {
        /** @type {?} */
        const content = this.contentElementRef.nativeElement;
        /** @type {?} */
        const firstOffsetTop = (content.querySelector('mark') || {}).offsetTop || 0;
        this.scrollSmooth(content, firstOffsetTop);
    }
    /**
     * @param {?} scrollElement
     * @param {?} firstOffsetTop
     * @return {?}
     */
    scrollSmooth(scrollElement, firstOffsetTop) {
        /** @type {?} */
        const renderer = this.renderer;
        if (cancelAnimationId) {
            cancelAnimationFrame(cancelAnimationId);
        }
        /** @type {?} */
        const currentScrollTop = scrollElement.scrollTop;
        /** @type {?} */
        const delta = firstOffsetTop - currentScrollTop;
        animate({
            duration: 500,
            /**
             * @param {?} timeFraction
             * @return {?}
             */
            timing(timeFraction) {
                return timeFraction;
            },
            /**
             * @param {?} progress
             * @return {?}
             */
            draw(progress) {
                /** @type {?} */
                const nextStep = currentScrollTop + progress * delta;
                renderer.setProperty(scrollElement, 'scrollTop', nextStep);
            }
        });
    }
}
MarkjsHighlightDirective.decorators = [
    { type: Directive, args: [{
                selector: '[markjsHighlight]'
            },] }
];
/** @nocollapse */
MarkjsHighlightDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
MarkjsHighlightDirective.propDecorators = {
    markjsHighlight: [{ type: Input }],
    markjsConfig: [{ type: Input }],
    minimumMarkLength: [{ type: Input }],
    scrollToFirstMarked: [{ type: Input }],
    getInstance: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    MarkjsHighlightDirective.prototype.markjsHighlight;
    /** @type {?} */
    MarkjsHighlightDirective.prototype.markjsConfig;
    /** @type {?} */
    MarkjsHighlightDirective.prototype.minimumMarkLength;
    /** @type {?} */
    MarkjsHighlightDirective.prototype.scrollToFirstMarked;
    /** @type {?} */
    MarkjsHighlightDirective.prototype.getInstance;
    /** @type {?} */
    MarkjsHighlightDirective.prototype.markInstance;
    /**
     * @type {?}
     * @private
     */
    MarkjsHighlightDirective.prototype.contentElementRef;
    /**
     * @type {?}
     * @private
     */
    MarkjsHighlightDirective.prototype.renderer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxMarkjsModule {
}
NgxMarkjsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MarkjsHighlightDirective],
                imports: [],
                exports: [MarkjsHighlightDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxMarkjsModule, MarkjsHighlightDirective as ɵa };
//# sourceMappingURL=matej.cupka-ngx-markjs.js.map
